#!/usr/bin/bash

# Set proper PATH
export PATH=/usr/bin:/usr/sbin

# Help text
if [ "x$1" == "x-h" -o "x$1" == "x--help" ]; then
    echo "Usage: $0 [-h|--help]"
    echo
    echo "Lists instances and their mdev devices on this hypervisor."
    echo "Only uses info from 'virsh dumpxml <instance>'."
    exit 0
fi

# Print header
printf "%-38s %-19s %-38s %-20s %-16s %s\n" \
       ' MDEV_DEVICE' ' INSTANCE' ' UUID' ' NAME' ' FLAVOR' ' PROJECT'

# Loop through instances and list results
for instance in $(virsh list --all --name); do

    eval $( virsh dumpxml $instance | \
		xmlstarlet sel -N nova='http://openstack.org/xmlns/libvirt/nova/1.0' -T -t \
			   -m '//domain' \
			   -o 'MDEV_DEVICE=' -v 'devices/hostdev[@type="mdev"]/source/address/@uuid' -n \
			   -o 'INSTANCE=' -v 'name' -n \
			   -o 'UUID=' -v 'uuid' -n \
			   -m 'metadata/nova:instance' \
			   -o 'NAME=' -v 'nova:name' -n \
			   -o 'FLAVOR=' -v 'nova:flavor/@name' -n \
			   -o 'PROJECT=' -v 'nova:owner/nova:project' -n
	    )

    printf "%-38s %-19s %-38s %-20s %-16s %s\n" \
	   $MDEV_DEVICE $INSTANCE $UUID $NAME $FLAVOR $PROJECT

done
