#!/usr/bin/bash

# Set proper PATH
export PATH=/usr/bin:/usr/sbin

# Our nvidia type. Configure in /etc/sysconfig/create-mdev
NVIDIA_GPU_TYPE=${NVIDIA_GPU_TYPE:='nvidia-52'}

# Get allocations
source /root/openrc
source /root/proxy.sh

# Create mapping of resource IDs and PCI-IDs for vGPU devices according to placement
# Resulting array: [ resource_id, pci_id ]
resources=()
for resource in $(openstack resource provider list -c uuid -c name -f value | grep $(hostname)_pci | sed "s/$(hostname)_pci_//; s/_/:/g; s/ /,/;"); do
    resources+=( $resource )
done

# Create mapping of instance IDs and PCI-IDs according to placement
# Resulting array: [ instance_id, pci_id ]
placement_instance_pci=()
for resource_tuple in ${resources[@]}; do
    resource_uuid=$(echo $resource_tuple | cut -d, -f1)
    resource_pciid=$(echo $resource_tuple | cut -d, -f2)
    for instance_id in $(openstack resource provider show --allocations $resource_uuid -c allocations -f json | jq -r '.allocations | keys[]'); do
	placement_instance_pci+=( "$instance_id,$resource_pciid" )
    done
done

# Create mapping of instance IDs and MDEV UUIDs for existing instances according to virsh
# Resulting array: [ instance_id, mdev_uuid ]
virsh_instance_mdev=()
for instance_id in $(virsh list --all --uuid); do
    virsh_mdev_uuid=$(virsh dumpxml $instance_id | xmlstarlet sel -t -v '/domain/devices/hostdev[@type="mdev"]/source/address/@uuid')
    virsh_instance_mdev+=( "$instance_id,$virsh_mdev_uuid" )
done

# Create mapping with PCI ID and MDEV UUID
# Resulting array: [ vgpu_pci_id, mdev_uuid ]
final_tuple=()
for mdev_tuple in ${virsh_instance_mdev[@]}; do
    instance_id_virsh=$(echo $mdev_tuple | cut -d, -f1)
    for instance_tuple in ${placement_instance_pci[@]}; do
	instance_id_placement=$(echo $instance_tuple | cut -d, -f1)
	if [ $instance_id_placement == $instance_id_virsh ]; then
	    this_pci_id=$(echo $instance_tuple | cut -d, -f2)
	    this_mdev_uuid=$(echo $mdev_tuple | cut -d, -f2)
	    final_tuple+=( "$this_pci_id,$this_mdev_uuid" )
	fi
    done
done

# Loop through and create MDEV devices existing instances, using the
# correct vGPU PCI ID according to placement
for final in ${final_tuple[@]}; do
    final_pciid=$(echo $final | cut -d, -f1 | sed -E 's/:([[:digit:]])$/.\1/')
    final_mdev_uuid=$(echo $final | cut -d, -f2)
    echo "$final_mdev_uuid" > /sys/class/mdev_bus/$final_pciid/mdev_supported_types/${NVIDIA_GPU_TYPE}/create
done
