#!/usr/bin/bash

# Set proper PATH
export PATH=/usr/bin:/usr/sbin

USAGE="Usage: $0 [OPTION...]"
HELP=$'
OPTIONS:
  -n            Dryrun, do not actually do anything
  -h            This help text
'

DRYRUN='no'

# Getting options
while getopts 'hn' opt; do
    case $opt in
        h)
            echo $USAGE
            echo "$HELP"
            exit 0
            ;;
        n)
            DRYRUN='yes'
            ;;
        \?)
            echo >&2 "Invalid option: -$OPTARG"
            exit 1
            ;;
        :)
            echo >&2 "Option -$OPTARG requires an argument."
            exit 1
    esac
done
shift $(($OPTIND - 1))

# find our mdev devices
mdev=()
for entry in $(ls /sys/class/mdev_bus); do
    mdev+=($entry)
done

# find and store mdev UUIDs in use by instances
uuids_in_use=()
for instance in $(virsh list --all --name); do
    instance_uuid=$(virsh dumpxml $instance | xmlstarlet sel -t -v '/domain/devices/hostdev[@type="mdev"]/source/address/@uuid')
    uuids_in_use+=( $instance_uuid )
    if [ "x$instance_uuid" == "x" ]; then
	echo >&2 "ERROR: Instance $instance does not have an mdev device"
    fi
done

# find and store mdev UUIDs in use according to mdevctl
uuids_by_mdevctl=()
for item in "$(mdevctl list | awk '{print $1}')"; do
    uuids_by_mdevctl+=( $item )
done

# find and store mdev UUIDs that are reported by mdevctl, but are not
# used by instances
uuids_to_delete=( ${uuids_by_mdevctl[@]} )
for delete in ${uuids_in_use[@]}; do
    uuids_to_delete=( "${uuids_to_delete[@]/$delete}" )
done

# remove unused devices
for uuid in ${uuids_to_delete[@]}; do
    if [ $DRYRUN == 'yes' ]; then
	echo "DRYRUN: Not deleting unused mdev UUID: $uuid"
    else
	echo 1 > /sys/bus/mdev/devices/$uuid/remove
	echo "Deleted unused mdev UUID: $uuid"
    fi
done
